﻿



using CoApp.Toolkit;

namespace CoApp.mkPackage
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Xml;
    using System.IO;
    using CoApp.Toolkit.Extensions;
    using CoApp.Toolkit.Package;
    using ctp = CoApp.Toolkit.Package;



    internal class IDizer
    {
        public const string schemaURL = "http://coapp.org/ns/mkPackage.xsd";


       // private XmlDocument doc = null;
        private Package p;
        private readonly List<string> componentIds = new List<string>();
        private readonly List<Role> roles = new List<Role>();
        private readonly List<IFileType> outerFiles = new List<IFileType>();
        private readonly List<IFileType> files = new List<IFileType>();
        private readonly List<string> binExts = new List<string> { ".exe", ".dll", ".ocx", ".vxd", ".lib" };
        private readonly HashSet<ctp.Folder> folders = new HashSet<ctp.Folder>();

      //  private const string roleXPath = "/in:Package/in:SharedLib|/in:Package/in:DeveloperLib|/in:Package/in:SourceCode|/in:Package/in:AppRole";
        //private const string fileXPath = "//in:Assembly|//in:Manifest|//in:CatFile|//in:File|//in:LibFile|//in:ExeFile";

        //this is needed because namespace support in System.Xml makes little to no sense
       // private XmlNamespaceManager nsMgr = null;

        #region Properties
        private string PackageName
        {
            get { return p.Name; }
        }

        private string PackageVersion
        {
            get { return p.Version; }
        }

        private string PackageArch
        {
            get { return p.Arch.ToString(); }
        }

        private string PackagePubToken
        {
            get { return p.Publisher.PublicKeyToken; }
        }

        #endregion
        /*
        internal IDizer(XmlDocument doc)
        {
            this.doc = doc;

            var nt = doc.NameTable;

            nsMgr = new XmlNamespaceManager(nt);

            nsMgr.AddNamespace("in", schemaURL);
        
        }
        */
        internal IDizer(Package p)
        {
            this.p = p;

            //load all the roles
            roles.AddRange(p.SharedLib);
            roles.AddRange(p.AppRole);
            roles.AddRange(p.DeveloperLib);
            roles.AddRange(p.DriverRole);


            foreach (var r in roles)
            {
                files.AddRange(getFiles(r));
                outerFiles.AddRange(getFiles(r, true));
            }
        }

        internal bool addIDs()
        {
            //componentIds = this.componentIds;
            try
            {

                addPackageGuid();
                addProductGuid();
                addRoleGuidsAndTypes();
                addTagGuids();
                performContributorModifications();
                performPublisherModifications();
                performInstallPropertyModifications();
                addUrlGuids();
                addLicenseGuids();
                addInstallPropertyGuids();
                addDependencyGuids();
                

                //next line MUST happen before the stuff after it
                addFilenameAndExts();


                addAssemblyGuidsAndIds();
                addPolicyAssemblyGuidsAndIds();
                addFileGuidsAndIds();
                addFolderIds();

                addPublisherId();

                addPackageDirectoryId();
                addIconId();

                
                
                AddFeatures();
            }
            catch (Exception)
            {
                return false;
            }
            return true;
        }

       


       

       
        

        private void addUrlGuids()
        {
            foreach (var u in p.Urls)
            {
                var type = u.Type;
                var value = u.URL;

                if (String.IsNullOrEmpty(u.url_guid))
                    u.url_guid = (string)new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "url", type, value);
            }
        }


        /*
        private void addUrlGuids()
        {
            foreach (XmlElement e in SelectNodes(doc, "/in:Package/in:Urls/in:Url"))
            {
                var type = SelectSingleNode(e, "@Type").Value;
                var value = SelectSingleNode(e, "@URL").Value;

                var tagHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "url", type, value);
                addAttrib("url_guid", (string)tagHuid, e);
            }
        }*/

        private void addDependencyGuids()
        {
            foreach (var d in p.Dependencies)
            {
                d.package_id = (string) new Huid(d.Name, d.Version, d.Arch.ToString(), d.PublicKeyToken);
            }
        }

        /*
        private void addDependencyGuids()
        {
            var dependencies = SelectNodes(doc, "/in:Package/in:Dependencies/in:Dependency");
            foreach (XmlElement e in dependencies)
            {
                var name = e.GetAttribute("Name");
                var arch = e.GetAttribute("Arch");
                var version = e.GetAttribute("Version");
                var pubToken = e.GetAttribute("PublicKeyToken");

                var h = new Huid(name, version, arch, pubToken);
                
                addAttrib("package_id", h.ToString(), e);
            }
        }*/

        private void addPackageDirectoryId()
        {
            p.proddir_id = (p.Name + "-" + p.Version + "-" + p.Arch).MakeSafeDirectoryId();
        }

        /*
        private void addPackageDirectoryId()
        {
            var packElem = (XmlElement)SelectSingleNode(doc, "/in:Package");
            packElem.SetAttribute("proddir_id", (packElem.GetAttribute("Name") + "-" + packElem.GetAttribute("Version") + "-" + 
                                packElem.GetAttribute("Arch")).MakeSafeDirectoryId()); 
          

        }*/

        private void addIconId()
        {
            if (p.Properties != null && p.Properties.Icon != null)
            {

                p.Properties.IconValueBase64 = Convert.ToBase64String(System.IO.File.ReadAllBytes(p.Properties.Icon));


            }
            else
            {
                if (p.Properties == null)
                    p.Properties = new Toolkit.Package.Properties();

                 p.Properties.IconValueBase64 = Convert.ToBase64String(Properties.Resources.blankicon);
            }
        }

        /*
        private void addIconId()
        {
            var propsObj = SelectSingleNode(doc, "/in:Package/in:Properties");
            if (propsObj != null)
            {
                var attrib = ((XmlElement)propsObj).GetAttribute("Icon");
                if (attrib != null)
                {
                    var packElem = (XmlElement)SelectSingleNode(doc, "/in:Package");
                    /// TODO: This may not be safe enough because something could have folder with this name?
                    addAttrib("icon_id", (packElem.GetAttribute("Name") + "-" + packElem.GetAttribute("Version") + "-" +
                                packElem.GetAttribute("Arch") + "-" + "___packageicon").MakeSafeDirectoryId(), (XmlElement)propsObj);
                }
            }
        }*/

        private void addPublisherId()
        {
            p.Publisher.directory_id = p.Publisher.Name.MakeSafeDirectoryId();
        }

        /*
        private void addPublisherId()
        {
            var pubElem = (XmlElement)SelectSingleNode(doc, "/in:Package/in:Publisher");
            pubElem.SetAttribute("directory_id", pubElem.GetAttribute("Name").MakeSafeDirectoryId());
        }*/
        /*
        private void addAttrib(string attribName, string attribVal, string pathToParent)
        {
            addAttrib(attribName, attribVal, (XmlElement)SelectSingleNode(doc, pathToParent));
        }
       
        private void addAttrib(string attribName, string attribVal, XmlElement e)
        {
            XmlAttribute attrib = doc.CreateAttribute(attribName);
            attrib.Value = attribVal;
            e.Attributes.Append(attrib);
        }

         */
        private void addPackageGuid()
        {
            p.package_guid = (string) new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken);
        }

        /*
        private void addPackageGuid()
        {
            addAttrib("package_guid", (string)new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken), "/in:Package");
        }*/

        private void addProductGuid()
        {
            p.product_guid = (string)new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "product");
        }

        /*
        private void addProductGuid()
        {
            addAttrib("product_guid", (string)new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "product"), "/in:Package");
        }*/

        private void addRoleGuidsAndTypes()
        {
          
            
            var flavor = "";
            foreach (var r in roles)
            {
                r.type = r.GetType().Name;
                flavor = "";
                if (r.Flavor != null)
                    flavor = r.Flavor;
                r.role_guid =
                    (string)
                    new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "role", r.type, flavor);
            }
        }

        /*
        private void addRoleGuidsAndTypes()
        {
            foreach (XmlElement e in SelectNodes(doc, roleXPath))
            {
                e.SetAttribute("type", e.LocalName);
                var flavor = "";
                var n = e.SelectSingleNode("@Flavor");
                if (n != null)
                    flavor = n.Value;

                var roleHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "role", e.LocalName, flavor);

                addAttrib("role_guid", (string)roleHuid, e);
            }
        }*/

        private void addTagGuids()
        {
            foreach (var t in p.Tags)
            {
                var type = t.Type;
                var value = t.Value;
                t.tag_guid = (string)new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "tag", type, value);
            }
        }

        /*
        private void addTagGuids()
        {
            foreach (XmlElement e in SelectNodes(doc, "/in:Package/in:Tags/in:Tag"))
            {
                var type = SelectSingleNode(e, "@Type").Value;
                var value = SelectSingleNode(e, "@Value").Value;

                var tagHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "tag", type, value);
                addAttrib("tag_guid", (string)tagHuid, e);
            }
        }*/
        

        private void addLicenseGuids()
        {
            foreach (var r in roles)
            {
                var parentHuid = r.role_guid;

                r.License.license_guid = (string)new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "license", parentHuid);
            }
        }

        /*
        private void addLicenseGuids()
        {
            foreach (XmlElement e in SelectNodes(doc, "//in:License"))
            {
                var parentHuid = e.ParentNode.Attributes["role_guid"].Value;

                var licenseHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "license", parentHuid);
                addAttrib("license_guid", (string)licenseHuid, e);
            }
        }*/


        private void addInstallPropertyGuids()
        {

            foreach (var r in roles)
            {
                foreach (var prop in r.InstallProperties)
                {
                    var parentHuid = r.role_guid;
                    var type = prop.Type;
                    var link = prop.Link;
                    var target = prop.Target;

                    prop.prop_guid = (string)new Huid(PackageName, PackageVersion,
                    PackageArch, PackagePubToken, "property", parentHuid, type, link, target);
                }
            }
        }

        /*
        private void addInstallPropertyGuids()
        {
            foreach (XmlElement e in SelectNodes(doc, "//in:InstallProperties/in:Property"))
            {
                var parentHuid = e.ParentNode.ParentNode.Attributes["role_guid"].Value;
                var type = e.Attributes["Type"].Value;
                var link = e.Attributes["Link"].Value;
                var target = e.Attributes["Target"].Value;

                var propHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, "property", parentHuid, type, link, target);
                addAttrib("prop_guid", (string)propHuid, e);
            }
        }*/

        private void addAssemblyGuidsAndIds()
        {
            var assms = from r in roles
                        where r.GetType() == typeof (SharedLib)
                        from a in ((SharedLib) r).Files.Assembly
                        select a;
                
            foreach (var a in assms)
            {
                var src = a.Src;

                var assmHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, src);
                a.component_guid = (string) assmHuid;

                var compId = CreateValidIdStr(assmHuid);
                componentIds.Add(compId);
                a.component_id = compId;

                //we also add in proper Type, very simple to do

                if (a.Arch == Arch.x86 || a.Arch == Arch.x64)
                {
                    a.type = "win32";
                }

                else if (a.Arch == Arch.any )
                {
                    a.type = ".net";
                }
            }
        }

        /*
        private void addAssemblyGuidsAndIds()
        {

      
            foreach (XmlElement e in SelectNodes(doc, "//in:Assembly"))
            {
                var src = SelectSingleNode(e,"@Src").Value;



                var assmHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, src);
                addAttrib("component_guid", (string)assmHuid, e);

                var compId = CreateValidIdStr(assmHuid);
                componentIds.Add(compId);
                addAttrib("component_id", compId, e);


                //we also add in proper Type, very simple to do

                if (e.Attributes["Arch"].Value == "x86" || e.Attributes["Arch"].Value == "x64")
                {
                    e.SetAttribute("type", "win32");
                }
                else if (e.Attributes["Arch"].Value == "any" && e.Attributes["Platform"].Value == "MSIL")
                {
                    //its msil
                    e.SetAttribute("type", ".net");
                }

            }
        }*/

        //TODO: Policy assemblies only really work on native assemblies, not .NET

        private void addPolicyAssemblyGuidsAndIds()
        {
            var sl = roles.OfType<SharedLib>();
            var pas = from s in sl
                     from pa in s.Files.PolicyAssembly
                     select pa;
            
            foreach (var pa in pas)
            {
                Huid assmHuid;
                if (pa.Arch == Arch.any)
                {
                    pa.type = ".net";
                    assmHuid =  new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, pa.Src);
                   
                }
                else
                {
                    pa.type = "win32";
                    assmHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, pa.Manifest.Src);

                }

                pa.component_guid = (string)assmHuid;

                var compId = CreateValidIdStr(assmHuid);
                componentIds.Add(compId);
                pa.component_id = compId;

              
            }

        }

        /*
        /// <summary>
        /// Policy Assemblies are given Guids and Ids. Additionally their types are set.
        /// </summary>
        private void addPolicyAssemblyGuidsAndIds()
        {
            foreach (XmlElement e in SelectNodes(doc, "//in:PolicyAssembly"))
            {


                var src = (from XmlElement child in e.ChildNodes
                          where child.LocalName == "Manifest"
                          select child.Attributes["Src"].Value).FirstOrDefault();

                var assmHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, src);
                addAttrib("component_guid", (string)assmHuid, e);

                var compId = CreateValidIdStr(assmHuid);
                componentIds.Add(compId);
                addAttrib("component_id", compId, e);


                //we also add in proper Type, very simple to do

                if (e.Attributes["Arch"].Value == "x86" || e.Attributes["Arch"].Value == "x64")
                {
                    e.SetAttribute("type", "win32");
                }
                else if (e.Attributes["Arch"].Value == "any" && e.Attributes["Platform"].Value == "MSIL")
                {
                    //its msil
                    e.SetAttribute("type", ".net");
                }

            }
        }*/
        
        //TODO: Why does the old version of this method check for ARCH?
        private void addFileGuidsAndIds()
        {
            foreach (var f in outerFiles.Cast<FileType>())
            {
                var fileHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, f.Src);
                f.component_guid = (string) fileHuid;

                var compid = CreateValidIdStr(fileHuid);
                componentIds.Add(compid);
                f.component_id = compid;
            }
        }

        /*
        /// <summary>
        /// Every file that doesn't belong to an assembly is it's own component so it's given
        /// a component huid and id.
        /// </summary>
        private void addFileGuidsAndIds()
        {
            //its a file BUT NOT in an assembly
            foreach (XmlElement e in getIndependentFiles())
            {

                var src = SelectSingleNode(e, "@Src").Value;
                Huid fileHuid;
                if (e.HasAttribute("Arch"))
                {
                    //use the local arch for the attribute
                    fileHuid = new Huid(PackageName, PackageVersion, e.Attributes["Arch"].Value, PackagePubToken, src);
                }
                else
                {
                    fileHuid = new Huid(PackageName, PackageVersion, PackageArch, PackagePubToken, src);
                }
                addAttrib("component_guid", (string)fileHuid, e);


                var compId = CreateValidIdStr(fileHuid);
                componentIds.Add(compId);
                addAttrib("component_id", compId, e);
            }

        }*/


        private void addFilenameAndExts()
        {
            foreach (var f in files)
            {
                var fi = new FileInfo(f.SrcProp);
                f.NameProp = fi.NameWithoutExt();
                f.ExtProp = fi.Extension;
            }
        }

        /*
        /// <summary>
        /// The main mkPackage program needs each file's 
        /// filename split from it's extension so we do that here.
        /// </summary>
        private void addFilenameAndExts()
        {
            foreach (XmlElement e in SelectNodes(doc, fileXPath))
            {
                var filePath = e.Attributes["Src"].Value;
                FileInfo fi = new FileInfo(filePath);
                addAttrib("name", fi.NameWithoutExt(), e);

                addAttrib("ext", fi.Extension, e);

            }
        }*/
        
        private void addFolderIds()
        {
            foreach (var r in roles)
            {
                calcFolderCompIds(r);
            }
        }

        private void performContributorModifications()
        {
            foreach (var c in p.Contributors)
            {

                if (!String.IsNullOrEmpty(c.Url))
                {
                    c.url_id = p.AddUrl(c.Url, "contributor");
                }
                
                c.id =
                    (string)
                    new Huid(p.Name, p.Version, p.Arch.ToString(), p.Publisher.PublicKeyToken, c.Name, c.Email, c.Url);
            }
        }


        private void performPublisherModifications()
        {
            if (!String.IsNullOrEmpty(p.Publisher.Location))
            {
                p.Publisher.location_id = p.AddUrl(p.Publisher.Location, "publisher_location");
            }
        }

        private void performInstallPropertyModifications()
        {
            
            if (!String.IsNullOrEmpty(p.Properties.Location))
            {
                p.Properties.location_id = p.AddUrl(p.Properties.Location, "original_location");
            }
            
            if (!String.IsNullOrEmpty(p.Properties.Feed))
            {
                p.Properties.feed_id = p.AddUrl(p.Properties.Feed, "feed_location");
            }
        }

        /*
        /// <summary>
        /// Folders need Msi Ids so we give each of them one.
        /// </summary>
        private void addFolderIds()
        {
            //also do the Headers include directory
            foreach (XmlElement e in SelectNodes(doc, "//in:Folder|//in:Headers"))
            {
                var folderId = getComponentId(e);
                addAttrib("folder_id", getComponentId(e), e);
            }
        }
        */
        /*
        private XmlNodeList getRoles()
        {
            return SelectNodes(doc, roleXPath);
        }*/

        /*
        private string getComponentId(Folder f)
        {
            if ()
        }*/

        /*
        private string getComponentId(XmlElement e)
        {
            if (getRoles().Cast<XmlElement>().Contains(e))
            {
                var flavor = "";
                if (e.HasAttribute("Flavor"))
                    flavor = e.Attributes["Flavor"].Value;
                return e.LocalName + flavor;
            }
            
            var output = getComponentId((XmlElement)e.ParentNode) + "___" + e.LocalName;
            switch (e.LocalName)
            {
                case "Assembly":

                case "LibFile":

                case "ExeFile":

                case "File":
                    output += "_" + e.Attributes["name"].Value + "_" + e.Attributes["ext"].Value;
                    break;
                case "Folder":
                    output += "_" + e.Attributes["Name"].Value;
                    break;
                case "Headers":
                    output += "__h_";
                    break;
            }


            return output.MakeSafeDirectoryId() ;
        }*/

      //private string getComponentId(Role r)

        private void calcFolderCompIds(ctp.Role r)
        {
            var flavor = "";
            if (r.Flavor != null)
                flavor = r.Flavor;
            var parentId = r.GetType().Name + flavor;

            if (r.GetType() == typeof(ctp.SharedLib))
            {
                var s = (SharedLib) r;
                foreach (var f in s.Files.Items)
                {
                    calcFolderCompIds(f, parentId);
                }
            }
            else if (r.GetType() == typeof(ctp.AppRole))
            {
                var a = (AppRole) r;
                
                foreach (var f in a.Files.Items)
                {
                    calcFolderCompIds(f, parentId);   
                }
            }
            else
            {
                Console.WriteLine("Those roles aren't supported!");
            }
        }
        


        private void calcFolderCompIds(ctp.FileSysItem item, string parentId)
        {
            var output = parentId + "___" + item.GetType().Name;
            //it's a file type
            /*
            if (item.GetType().IsSubclassOf(typeof(ctp.FileType)))
            {
                var f = (ctp.FileType) item;
                output += "_{0}_{1}".format(f.name, f.ext);
                
            }*/
            if (item.GetType() == typeof(ctp.Folder))
            {
                var f = (ctp.Folder) item;
                output += "_{0}".format(f.Name);
                f.folder_id = output.MakeSafeDirectoryId();
                foreach (var ch in f.Items)
                {
                    calcFolderCompIds(ch, f.folder_id);
                }
            }
                /*
            else if (item.GetType() == typeof(ctp.Headers))
            {
                var f = (ctp.Headers) item;

            }*/
        }
        /*
        internal XmlNode SelectSingleNode(XmlNode n, string xpath)
        {
            return n.SelectSingleNode(xpath, nsMgr);
        }

    
        internal XmlNodeList SelectNodes(XmlNode n, string xpath)
        {
            return n.SelectNodes(xpath, nsMgr);
        }

        */
        /*
        /// <summary>
        /// Gets all the files in the package that don't belong to an assembly or the independent files.
        /// </summary>
        /// <returns>an XmlNodeList containing all the nodes corresponding to the independent files</returns>
        private XmlNodeList getIndependentFiles()
        {
            //get immediate children of SharedLib
            string xpathStr = "/in:Package/in:SharedLib/in:Files/in:File";
            //get nonimmediate children of SharedLib
            xpathStr += "|/in:Package/in:SharedLib/in:Files/in:Folder//in:File";

            //get immediate children of DevLib
            xpathStr +=  "|/in:Package/in:DeveloperLib/in:Includes/in:File";
            //get nonimmediate children of DevLib
            xpathStr +=  "|/in:Package/in:DeveloperLib/in:Includes//in:File";
            //get LibFiles
            xpathStr += "|/in:Package/in:DeveloperLib/in:Includes//in:LibFile";
            xpathStr += "|/in:Package/in:DeveloperLib/in:Includes//in:LibFile";

            //get immediate children of SourceCode
            xpathStr +=  "|/in:Package/in:SourceCode/in:CodeFiles/in:File";
            //get nonimmediate children of SourceCode
            xpathStr +=  "|/in:Package/in:SourceCode/in:CodeFiles/in:Folder//in:File";

            xpathStr += "|/in:Package/in:AppRole/in:AppFiles/in:File";
            xpathStr += "|/in:Package/in:AppRole/in:AppFiles/in:ExeFile";
            xpathStr += "|/in:Package/in:AppRole/in:AppFiles/in:Folder//in:File";

            return SelectNodes(doc, xpathStr);
        }
        */
        

        /// <summary>
        /// Takes Huid and turns it into a valid Msi Id. May be the most evil code ever written.
        /// </summary>
        /// <param name="h">a Huid that you want to turn into an Msi Id</param>
        /// <returns>an Id string corresponding to h</returns>
        private static String CreateValidIdStr(Huid h)
        {
            return "s_" + ((string)h).MakeSafeDirectoryId().Replace("{","").Replace("}","");
        }

        
        private IEnumerable<ctp.IFileType> getFiles(Role r, bool indy = false)
        {
            if (r.GetType() == typeof(ctp.SharedLib))
                return getFiles((ctp.SharedLib)r, indy);
            else if (r.GetType() == typeof(ctp.AppRole))
                return getFiles((ctp.AppRole)r, indy);
            else if (r.GetType() == typeof(ctp.DeveloperLib))
                return getFiles((ctp.DeveloperLib)r, indy);
            else if (r.GetType() == typeof(ctp.DriverRole))
                return getFiles((ctp.DriverRole)r, indy);
            return new List<ctp.FileType>();
        }
        


        private IEnumerable<ctp.IFileType> getFiles(SharedLib r, bool indy = false)
        {
            var files = new List<ctp.IFileType>();
            
            foreach (var i in r.Files.Items)
            {
                if (i.GetType() == typeof(ctp.File))
                    files.Add((ctp.File)i);
                else if (i.GetType() == typeof(ctp.Folder))
                    files.AddRange(getFiles((ctp.Folder)i));
            }

            if (!indy)
            {
                files.AddRange(getFiles(r.Files.Assembly));
                files.AddRange(getFiles(r.Files.PolicyAssembly));
            }

            return files;
        }

        private IEnumerable<ctp.IFileType> getFiles(AppRole r, bool indy = false)
        {
            var files = new List<ctp.IFileType>();
            foreach (var i in r.Files.Items)
            {
                if (i.GetType() == typeof(ctp.File))
                    files.Add((ctp.File)i);
                else if (i.GetType() == typeof(ctp.Folder))
                    files.AddRange(getFiles((ctp.Folder)i));
            }
            files.AddRange(r.Files.ExeFile);
            return files;
        }

        private IEnumerable<ctp.IFileType> getFiles(DeveloperLib r, bool indy = false)
        {
            var files = new List<ctp.IFileType>();
            
            Console.WriteLine("DeveloperLib not supported.");
            return files;
        }

        private IEnumerable<ctp.IFileType> getFiles(DriverRole r, bool indy = false)
        {
            Console.WriteLine("DriverRole not supported.");
            return new List<ctp.IFileType>();
        }

        private IEnumerable<ctp.IFileType> getFiles(ctp.Folder f)
        {

            //create the folder list in a god awful way
            folders.Add(f);


            var files = new List<ctp.IFileType>();
            foreach (var i in f.Items)
            {
                if (i.GetType() == typeof(ctp.File))
                    files.Add((ctp.File)i);
                else if (i.GetType() == typeof(ctp.Folder))
                    files.AddRange(getFiles((ctp.Folder)i));
            }

            return files;
        }

        private IEnumerable<ctp.IFileType> getFiles(IEnumerable<ctp.Assembly> assemblies)
        {
            var files = new List<ctp.IFileType>();
            foreach (var a in assemblies)
            {
                files.AddRange(getFiles(a));
            }

            return files;
        }

        private IEnumerable<ctp.IFileType> getFiles(ctp.Assembly a)
        {
            var files = new List<ctp.IFileType>();
            //add the assembly
            files.Add(a);
            files.AddRange(a.Items);
            if (a.CatFile != null)
                files.Add(a.CatFile);
            if (a.Manifest != null)
                files.Add(a.Manifest);

            return files;
        }

        private IEnumerable<ctp.IFileType> getFiles(IEnumerable<ctp.PolicyAssembly> assemblies)
        {
            var files = new List<ctp.IFileType>();
            foreach (var a in assemblies)
            {
                files.AddRange(getFiles(a));
            }

            return files;
        }

        private IEnumerable<ctp.IFileType> getFiles(ctp.PolicyAssembly a)
        {
            var files = new List<ctp.IFileType>();
            //add the assembly
            if (!String.IsNullOrEmpty(a.Src))
            {
                files.Add(a);
            }
           // files.Add(a);
            //files.AddRange(a.Items);
            if (a.CatFile != null)
                files.Add(a.CatFile);
            if (a.Manifest != null)
                files.Add(a.Manifest);

            return files;
        }

        //TODO: doesn't get cat files
        internal IEnumerable<string> CollectAssmFiles()
        {
            var sharedlibs = from role in roles where role.GetType() == typeof (ctp.SharedLib) select (ctp.SharedLib)role;
            
            var assms =  from s in sharedlibs  from a in s.Files.Assembly select a;
            //TODO: catfiles can't be checked for signing
            //var catfiles = from a in assms select a.CatFile.Src;
            
            var binUnderAssms = from a in assms
                                from f in a.Items
                                where binExts.Any(ext => f.Src.EndsWith(ext))
                                select f.Src;

            var policyAssms = from s in sharedlibs from a in s.Files.PolicyAssembly select a;

            //var binUnderPolicyAssms = from a in policyAssms
                                      

            return binUnderAssms.Concat(from a in assms select a.Src).Concat(from a in policyAssms where !String.IsNullOrEmpty(a.Src) select a.Src);

        }
        

        internal IEnumerable<string> CollectBinFiles()
        {
            var sharedlibs = from role in roles where role.GetType() == typeof (ctp.SharedLib) select (ctp.SharedLib)role;
            var approles = from role in roles where role.GetType() == typeof (ctp.AppRole) select (ctp.AppRole) role;

            var slfiles = from s in sharedlibs
                          let bins = CollectBinFiles(s.Files.Items)
                          from f in bins
                          select f;
                         
            var arfiles = from a in approles
                          let bins = CollectBinFiles(a.Files.Items)
                          from f in bins
                          select f;

            var exefiles = from a in approles
                           from e in a.Files.ExeFile
                           select e.Src;
            
            return slfiles.Concat(arfiles).Concat(exefiles);
        }


        private IEnumerable<string> CollectBinFiles(IEnumerable<FileSysItem> items)
        {
            var files = from f in items
                        where f.GetType() == typeof(ctp.File)
                        let src = ((ctp.File)f).Src
                        where binExts.Any(ext => src.EndsWith(ext))
                        select src;

            //these could be files and children
            var childItems = from f in items
                          where f.GetType() == typeof (ctp.Folder)
                          let folder = (ctp.Folder) f
                          from i in folder.Items
                          select i;


            if (childItems.Count() == 0)
                return files;

            return files.Concat(CollectBinFiles(childItems));
        }
      

        internal void AddFeatures()
        {
            if (p.features == null)
                p.features = new List<comp>();

            foreach (var compid in componentIds)
            {
                p.features.Add(new ctp.comp() {component_id = compid});
            }
        }

    }
}
