﻿

namespace CoApp.mkPackage
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Mono.Security.X509;
    using Mono.Security.Authenticode;
    using System.IO;
    using System.Security.Cryptography.X509Certificates;
    using Mono.Security;
    using CoApp.Toolkit.Extensions;

    internal class PKCSVerifier : SigVerifier
    {
        private string pfxFile = null;
        private string password = null;

        internal PKCSVerifier(string pfxFile)
        {
            this.pfxFile = pfxFile;
        }

        internal PKCSVerifier(string pfxFile, string password)
        {
            this.pfxFile = pfxFile;
            this.password = password;
        }


        internal override bool VerifyAssemblies(IEnumerable<string> assemblies)
        {
            bool ret = true;

            PKCS12 pkcs = null;
            //first open pfxfile
            try
            {
                if (password == null)
                {
                    pkcs = PKCS12.LoadFromFile(pfxFile);
                }
                else
                {
                    pkcs = PKCS12.LoadFromFile(pfxFile, password);
                }

            }
            catch
            {
                Console.WriteLine("Something went wrong when you tried to open the PKCS file");
                return false;
            }

            var codeSigningCert = pkcs.FindCodeSigningCert();
            byte[] pubKey = codeSigningCert.PublicKey;
            var expiryDate = codeSigningCert.ValidUntil;
            foreach (string s in assemblies)
            {
                try
                {
                    var authenticodeDeformatter = new AuthenticodeDeformatter(Path.GetFullPath(s));
                    var fileCert = authenticodeDeformatter.SigningCertificate;
                    //var test = new ASN1(fileCert.PublicKey);
                    if (fileCert == null || !pubKey.SequenceEqual(fileCert.PublicKey) || expiryDate != fileCert.ValidUntil)
                    {
                        ret = false;
                        Console.WriteLine("The file " + s + " was not signed with the same certificate that was passed in." );
                    }

                }
                catch (Exception e)
                {
                    ret = false;
                    Console.WriteLine(e.Message);
                }
            }

            return ret;
        }
    }
}
