﻿namespace CoApp.mkPackage
{
    using System;
    using Mono.Security.X509;
    using CoApp.Toolkit.Utility;
    using System.IO;

    public class PfxSigner : FileSigner
    {
        private string pfxFile = null;
        private string password = null;
        private PKCS12 pfxCert = null;

        public override PKCS12 PkcsFile
        {
            get
            {
                if (pfxCert == null)
                {
                    try
                    {
                        pfxCert = password == null ? PKCS12.LoadFromFile(pfxFile) : PKCS12.LoadFromFile(pfxFile, password);
                    }
                    catch
                    {
                        return null;
                    }
                    
                }
                return pfxCert;
            }
        }

        public PfxSigner(string pfxFile)
        {
            this.pfxFile = pfxFile;
        }
        public PfxSigner(string pfxFile, string password)
        {
            this.pfxFile = pfxFile;
            this.password = password;
        }

        public override bool Sign(string file)
        {

            var p = new ProcessUtility(ProgramFinder.ProgramFilesAndDotNet.ScanForFile("signtool.exe"));

            var ret = p.Exec(@"sign /a /f ""{0}"" /p {1} /t {2} ""{3}""", pfxFile, password, "http://timestamp.verisign.com/scripts/timstamp.dll", Path.GetFileName(file));
    


            if (ret != 0)
            {
                Console.WriteLine(p.StandardError);
                return false;
            }

            return true;

        }
        
    }
}
