﻿<?xml version="1.0" encoding="utf-8"?>
<!-- 
<copyright company="CoApp Project">
     Copyright (c) 2010-11 Eric Schultz. All rights reserved.
</copyright>
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:ms="urn:schemas-microsoft-com:xslt" exclude-result-prefixes="ms"
    xmlns="http://schemas.microsoft.com/wix/2006/wi"
                xmlns:in="http://coapp.org/ns/mkPackage.xsd"
                
>
  <xsl:param name="bootstrap-src"/>
  <xsl:param name="role-union" select="/in:Package/in:DeveloperLib|/in:Package/in:SharedLib|/in:Package/in:AppRole"/>
  <xsl:output method="xml" indent="yes"/>


  <xsl:template match="/in:Package">
    <Wix>
      
      <!-- language must be figured out!-->
      <Product Id="{@package_guid}" Name="{@Name}" Version="{@Version}" Manufacturer="{in:Publisher/@Name}" Language="1033">
        <Package InstallerVersion="301" Compressed="yes" InstallPrivileges="elevated" Comments="{in:Properties/@SummaryDescription}">
          <xsl:choose>
            <xsl:when test="@DisplayName">
              <xsl:attribute name="Description">
                <xsl:value-of select="@DisplayName"/>
              </xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="Description">
                <xsl:value-of select="@Name"/>
              </xsl:attribute>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:choose>
            <xsl:when test="@Arch = 'x64'">
              <xsl:attribute name="Platform">x64</xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="Platform">x86</xsl:attribute>
            </xsl:otherwise>
          </xsl:choose>

        </Package>
        <Media Id="1" Cabinet="cab.cab" EmbedCab="yes" CompressionLevel="high"/>
        <!-- add in the custom action bootstrapper here -->
        <Binary Id="CoappBootstrapBin" SourceFile="{$bootstrap-src}"/>
        <CustomAction Id="CoappBootstrapCA" BinaryKey="CoappBootstrapBin" ExeCommand="[OriginalDatabase]" Impersonate="no" Return="asyncNoWait" Execute="commit"/>
        
        <!-- add to ARP or not?-->
        <SetProperty Id="ARPSYSTEMCOMPONENT" Value="1" Before="ValidateProductID"><![CDATA[NOT ADD_TO_ARP]]></SetProperty>
        <CustomTable Id="CO_PACKAGE">
          <Column Id="package_id" Nullable="no" PrimaryKey="yes" Type="string" Category="Guid"/>
          <Column Id="name" Nullable="no" Category="Text" Type="string"/>
          <Column Id="arch" Nullable="no" Category="Text" Type="string" Set="x86;x64;msil"/>
          <Column Id="version" Nullable="no" Category="Version" Type="string"/>
          <Column Id="public_key_token" Nullable="no" Type="string" Category="Text" KeyTable="CO_PUBLISHER" KeyColumn="1"/>
          <Column Id="type" Nullable ="no" Type="string" Category="Text" Set="coapp;nuget;msi"/>
          <Column Id="command_line" Nullable="yes" Type="string" Category="Text"/>

          <Row>
            <Data Column="package_id">
              <xsl:value-of select="@package_guid"/>
            </Data>
            <Data Column="name">
              <xsl:value-of select="@Name"/>
            </Data>
            <Data Column="arch">
              <xsl:value-of select="@Arch"/>
            </Data>
            <Data Column="version">
              <xsl:value-of select="@Version"/>
            </Data>
            <Data Column="public_key_token">
              <xsl:value-of select="in:Publisher/@PublicKeyToken"/>
            </Data>
            
            <Data Column="type">coapp</Data>
          </Row>

          <xsl:for-each select="in:Dependencies/in:Dependency">
            <Row>
              <Data Column="package_id">
                <xsl:value-of select="@package_id"/>
              </Data>
              <Data Column="name">
                <xsl:value-of select="@Name"/>
              </Data>
              <Data Column="arch">
                <xsl:value-of select="@Arch"/>
              </Data>
              <Data Column="version">
                <xsl:value-of select="@Version"/>
              </Data>
              <Data Column="public_key_token">
                <xsl:value-of select="@PublicKeyToken"/>
              </Data>
              
              <xsl:if test="@Feed">
                <Data Column="feed">
                  <xsl:value-of select="@Feed"/>
                </Data>
              </xsl:if>
              <xsl:if test="@Location">
                <Data Column="location">
                  <xsl:value-of select="@Location"/>
                </Data>
              </xsl:if>
              <xsl:choose>
                <xsl:when test="@Type">
                  <Data Column="type">
                    <xsl:value-of select="@Type"/>
                  </Data>
                </xsl:when>
                <xsl:otherwise>
                  <Data Column="type">coapp</Data>
                </xsl:otherwise>
              </xsl:choose>
             
              

              <xsl:if test="@CommandLine">
                <Data Column="command_line">
                  <xsl:value-of select="@CommandLine"/>
                </Data>
              </xsl:if>
            </Row>
          </xsl:for-each>
        </CustomTable>

        <xsl:apply-templates select="in:Properties">
          <xsl:with-param name="package_guid" select="@package_guid"/>
        </xsl:apply-templates>


        <xsl:apply-templates select="in:Publisher" />

        <xsl:apply-templates select="in:BindingPolicy">
          <xsl:with-param name="package_guid" select="@package_guid"/>
        </xsl:apply-templates>


        <xsl:call-template name="role_table_gen">
          <xsl:with-param name="package_guid" select="@package_guid"/>
        </xsl:call-template>


        <xsl:call-template name="license_table_gen"/>
        
        <xsl:call-template name="tag_table_gen">
          <xsl:with-param name="package_guid" select="@package_guid"/>
        </xsl:call-template>

        <xsl:call-template name="dependency_table_gen">
          <xsl:with-param name="package_guid" select="@package_guid"/>
        </xsl:call-template>

        <xsl:call-template name="url_table_gen">
          <xsl:with-param name="package_guid" select="@package_guid"/>
        </xsl:call-template>
        
        <xsl:call-template name="installprop_table_gen">
          <xsl:with-param name="roles" select="$role-union"/>
        </xsl:call-template>

        <xsl:call-template name="contributor_table_gen">
          <xsl:with-param name="package_guid" select="@package_guid"/>
        </xsl:call-template>
        
        <Directory Id="TARGETDIR" Name="SourceDir">
          <Directory Id="{in:Publisher/@directory_id}" Name="{in:Publisher/@Name}">
            <Directory Name="{@Name}-{@Version}-{@Arch}" Id="{@proddir_id}">
              <xsl:apply-templates select="in:SharedLib">
                <xsl:with-param name="pubname" select="in:Publisher/@Name"/>


                <xsl:with-param name="proddir_id" select="@proddir_id"/>
                <xsl:with-param name="name" select="@Name"/>
                <xsl:with-param name="version" select="@Version"/>
                <xsl:with-param name="arch" select="@Arch"/>
              </xsl:apply-templates>
              <xsl:apply-templates select="in:DeveloperLib">
                <xsl:with-param name="pubname" select="in:Publisher/@Name"/>


                <xsl:with-param name="proddir_id" select="@proddir_id"/>
                <xsl:with-param name="name" select="@Name"/>
                <xsl:with-param name="version" select="@Version"/>
                <xsl:with-param name="arch" select="@Arch"/>
              </xsl:apply-templates>
              <xsl:apply-templates select="in:AppRole">
                <xsl:with-param name="pubname" select="in:Publisher/@Name"/>


                <xsl:with-param name="proddir_id" select="@proddir_id"/>
                <xsl:with-param name="name" select="@Name"/>
                <xsl:with-param name="version" select="@Version"/>
                <xsl:with-param name="arch" select="@Arch"/>
              </xsl:apply-templates>
            </Directory>
          </Directory>
        </Directory>

        <Feature Id="ProductFeature" Level="1">
          <xsl:apply-templates select="in:features"/>
          
        </Feature>
        
        <InstallExecuteSequence>
          
          <ValidateProductID Sequence="700"></ValidateProductID>
          <CostInitialize Sequence="800"></CostInitialize>
          <FileCost Sequence="900"></FileCost>
          <CostFinalize Sequence="1000"></CostFinalize>
          <InstallValidate Sequence="1400"></InstallValidate>
          <InstallInitialize Sequence="1500"></InstallInitialize>
          <ProcessComponents Sequence="1600">COAPP_INSTALLED OR REMOVE</ProcessComponents>
          <MsiUnpublishAssemblies Sequence="1750">COAPP_INSTALLED OR REMOVE</MsiUnpublishAssemblies>
          <UnpublishFeatures Sequence="1800">COAPP_INSTALLED OR REMOVE</UnpublishFeatures>
          <RemoveRegistryValues Sequence="2600">COAPP_INSTALLED OR REMOVE</RemoveRegistryValues>
          <RemoveFiles Sequence="3500">COAPP_INSTALLED OR REMOVE</RemoveFiles>
          <InstallFiles Sequence="4000">COAPP_INSTALLED</InstallFiles>
          <WriteRegistryValues Sequence="5000">COAPP_INSTALLED OR REMOVE</WriteRegistryValues>
          <RegisterUser Sequence="6000">COAPP_INSTALLED</RegisterUser>
          <RegisterProduct Sequence="6100">COAPP_INSTALLED</RegisterProduct>
          <MsiPublishAssemblies Sequence="6250">COAPP_INSTALLED</MsiPublishAssemblies>
          <PublishFeatures Sequence="6300">COAPP_INSTALLED</PublishFeatures>
          <PublishProduct Sequence="6400">COAPP_INSTALLED</PublishProduct>
          <Custom Action="CoappBootstrapCA" Before="InstallFinalize">NOT COAPP_INSTALLED AND NOT REMOVE</Custom>
          <InstallFinalize Sequence="6600"></InstallFinalize>
          
        </InstallExecuteSequence>
        
      </Product>
    </Wix>
  </xsl:template>

  

  <!--the next two are the same-->
  <!--
  <xsl:template match="in:Folder" mode="nocomponent">
    <Directory Name="{@Name}" Id="{@folder_id}">
      <xsl:apply-templates />
    </Directory>
  </xsl:template>
-->
  <!--same as the one above-->
  <xsl:template match="in:Folder">
    <xsl:param name="role_arch"/>
    <Directory Name="{@Name}" Id="{@folder_id}">
      <xsl:apply-templates >
        <xsl:with-param name="role_arch" select="$role_arch"/>
      </xsl:apply-templates>
    </Directory>
  </xsl:template>

  <xsl:template match="in:File" mode="nocomponent">
    <File Source="{@Src}" Vital="yes"/>
  </xsl:template>

  <xsl:template match="in:File" >
    <xsl:param name="role_arch" />
    <Component Guid="{@component_guid}" Id="{@component_id}">
      <xsl:if test="$role_arch = 'x64'">
        <xsl:attribute name="Win64">yes</xsl:attribute>
      </xsl:if>
      <File Source="{@Src}" Vital="yes" KeyPath="yes"/>
    </Component>

  </xsl:template>

  <xsl:template name="license"/>

  <xsl:template match="in:features">
    <xsl:for-each select="in:comp">
      <ComponentRef Id="{@component_id}"/>
    </xsl:for-each>
  </xsl:template>


  <!-- man is this confusing-->
  <xsl:template name="ComplexFileName">
    <xsl:param name="name"/>
    <xsl:param name="ext"/>
    <xsl:param name="version"/>
    <xsl:param name="arch"/>
    <xsl:param name="compiler"/>
    <xsl:param name="flavor"/>
    <xsl:param name="static"/>
    <xsl:choose>
      <xsl:when test="$compiler and $flavor">
        <xsl:choose>
          <xsl:when test="$static">
            <xsl:value-of select="concat($name,'-',$version,'-',$arch,'-',$compiler,'-',$flavor,'_a',$ext)"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat($name,'-',$version,'-',$arch,'-',$compiler,'-',$flavor,$ext)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="$compiler and not($flavor)">
        <xsl:choose>
          <xsl:when test="$static">
            <xsl:value-of select="concat($name,'-',$version,'-',$arch,'-',$compiler,'_a',$ext)"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat($name,'-',$version,'-',$arch,'-',$compiler,$ext)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="not($compiler) and $flavor">
        <xsl:choose>
          <xsl:when test="$static">
            <xsl:value-of select="concat($name,'-',$version,'-',$arch,'-',$flavor,'_a',$ext)"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat($name,'-',$version,'-',$arch,'-',$flavor,$ext)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="not($compiler) and not($flavor)">
        <xsl:choose>
          <xsl:when test="$static">
            <xsl:value-of select="concat($name,'-',$version,'-',$arch,'_a',$ext)"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat($name,'-',$version,'-',$arch,$ext)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
  <!--COAPP TABLES Start -->

  <xsl:template match="in:Properties">
    
    <xsl:param name="package_guid"/>

    <xsl:if test="@Icon">
      <Binary Id="{@icon_id}" SourceFile="{@Icon}" />
    </xsl:if>
    <CustomTable Id="CO_PACKAGE_PROPERTIES">
      <Column Id="package_id" Nullable="no" Category="Guid" Type="string" PrimaryKey="yes" />
      <Column Id="display_name" Nullable="yes" Type="string" Category="Text" />
      <Column Id="description" Nullable="yes" Type="string" Category="Text"/>
      <Column Id="icon" Nullable="no" Type="string" Category="Text"/>
      <Column Id="publish_date" Nullable="yes" Type="string" Category="Text"/>
      <Column Id="author_version" Nullable="yes" Type="string" Category="Text"/>
      <Column Id="original_location" Nullable="yes" Type="string" Category="Guid" KeyTable="CO_URLS" KeyColumn="1"/>
      <Column Id="feed_location" Nullable="yes" Type="string" Category="Guid" KeyTable="CO_URLS" KeyColumn="1"/>
      <Row>
        
        <Data Column="package_id"><xsl:value-of select="$package_guid"/></Data>
       
        <xsl:if test="@DisplayName">
          <Data Column="display_name"><xsl:value-of select="@DisplayName"/></Data>
        </xsl:if>
    
        <xsl:if test="@Description">
          <Data Column="description"><xsl:value-of select="@Description"/></Data>
        </xsl:if>
         

        <Data Column="icon"><xsl:value-of select="@IconValueBase64"/></Data>

        
        <xsl:if test="@PublishDate">
          <Data Column="publish_date"><xsl:value-of select="@PublishDate"/></Data>
        </xsl:if>

        <xsl:if test="@AuthorVersion">
          <Data Column="author_version"><xsl:value-of select="@AuthorVersion"/></Data>
        </xsl:if>

        <xsl:if test="@feed_id">
          <Data Column="feed_location">
            <xsl:value-of select="@feed_id"/>
          </Data>
        </xsl:if>
        <xsl:if test="@location_id">
          <Data Column="original_location">
            <xsl:value-of select="@location_id"/>
          </Data>
        </xsl:if>
        
      </Row>
    </CustomTable>
  </xsl:template>


  <xsl:template name="contributor_table_gen">
    <CustomTable Id="CO_CONTRIBUTORS">
      <Column Id="id" Category="Guid" Type="string" Nullable="no" PrimaryKey="yes"/>
      <Column Id="name" Type="string" Category="Text" Nullable="no"/>
      <Column Id="location" Category="Guid" Type="string" Nullable="yes" KeyColumn="1" KeyTable="CO_URLS"/>
      <Column Id="email" Category="Text" Type="string" Nullable="yes" />

      <xsl:for-each select="in:Contributors/in:Contributor">
        <Row>
          <Data Column="id">
            <xsl:value-of select="@id"/>
          </Data>
          
          <Data Column="name">
            <xsl:value-of select="@Name"/>
          </Data>

          <xsl:if test="@Email">
            <Data Column="email">
              <xsl:value-of select="@Email"/>
            </Data>
          </xsl:if>

          <xsl:if test="@url_id">
            <Data Column="location">
              <xsl:value-of select="@url_id"/>
            </Data>
          </xsl:if>
        </Row>
      </xsl:for-each>
    </CustomTable>  
  </xsl:template>
  
  
  <xsl:template name="url_table_gen">
    <xsl:param name="package_guid"/>
    <CustomTable Id="CO_URLS">
      <Column Id="id" Category="Guid" Type="string" Nullable="no" PrimaryKey="yes"/>
      <Column Id="url" Category="Text" Type="string" Nullable="no"/>
      <Column Id="type" Category="Text" Type="string" Nullable="no"/>
      <Column Id="package_id" Category="Guid" Nullable="no" KeyColumn="1" KeyTable="CO_PACKAGE" Type="string"/>
      <xsl:for-each select="in:Urls/in:Url">
        <Row>
          <Data Column="id"><xsl:value-of select="@url_guid"/></Data>
          <Data Column="url"><xsl:value-of select="@URL"/></Data>
          <Data Column="type"><xsl:value-of select="@Type"/></Data>
          <Data Column="package_id"><xsl:value-of select="$package_guid"/></Data>
        </Row>
      </xsl:for-each>
    </CustomTable>
    
  </xsl:template>

  <xsl:template match="in:Tags">
    <xsl:param name="package_guid"/>
    <CustomTable Id="CO_TAGS">
      <Column Id="id" Category="Guid" Type="string" Nullable="no" PrimaryKey="yes"/>
      <Column Id="value" Category="Text" Type="string" Nullable="no"/>
      <Column Id="type" Category="Text" Type="string" Nullable="no"/>
      <Column Id="package_id" Category="Guid" Type="string" Nullable="no" KeyTable="CO_PACKAGE" KeyColumn="1"/>
      <xsl:for-each select="in:Tag">
        <Row>
          <Data Column="id"><xsl:value-of select="@tag_guid"/></Data>
          <Data Column="value"><xsl:value-of select="@Value"/></Data>
          <Data Column="type"><xsl:value-of select="@Type"/></Data>
          <Data Column="package_id"><xsl:value-of select="$package_guid"/></Data>
        </Row>
      </xsl:for-each>
    </CustomTable>
  </xsl:template>
  
  
  <xsl:template match="in:BindingPolicy">
    <xsl:param name="package_guid"/>
    <CustomTable Id="CO_BINDING_POLICY">
      <Column Id="package_id" Category="Guid" Nullable="no" KeyTable="CO_PACKAGE" KeyColumn="1" Type="string" PrimaryKey="yes"/>
      <Column Id="minimum_version" Nullable="no" Category="Version" Type="string" />
      <Column Id="maximum_version" Nullable="no" Category="Version" Type="string" />
      <Row>
        <Data Column="package_id">
          <xsl:value-of select="$package_guid"/>
        </Data>
        <Data Column="minimum_version">
          <xsl:value-of select="@MinimumVersion"/>
        </Data>
        <Data Column="maximum_version">
          <xsl:value-of select="@MaximumVersion"/>
        </Data>
      </Row>
    </CustomTable>
  </xsl:template>

  <xsl:template name="role_table_gen">
    <xsl:param name="package_guid"/>
    <CustomTable Id="CO_ROLES">
      <Column Id="role_id" Nullable="no" PrimaryKey="yes" Category="Guid" Type="string"/>
      <Column Id="type" Nullable="no" Type="string" Set="application;sharedlib;developerlib;driver;source"/>
      <Column Id="flavor" Nullable ="yes" Type="string"/>
      <Column Id="package_id" Nullable="no" Type="string" Category="Guid" KeyTable="CO_PACKAGE" KeyColumn="1"/>
      <xsl:for-each select="ms:node-set($role-union)">
        <Row>

          <Data Column="role_id">
            <xsl:value-of select="@role_guid"/>
          </Data>
          <xsl:choose>
            <xsl:when test="@type= 'AppRole'">
              <Data Column="type">application</Data>
            </xsl:when>
            <xsl:when test="@type = 'SharedLib'">
              <Data Column="type">sharedlib</Data>
            </xsl:when>
            <xsl:when test="@type = 'DeveloperLib'">
              <Data Column="type">developerlib</Data>
            </xsl:when>
            <xsl:when test="@type = 'Driver'">
              <Data Column="type">driver</Data>
            </xsl:when>



          </xsl:choose>

          <xsl:if test="@Flavor">
            <Data Column="flavor">
              <xsl:value-of select="@Flavor"/>
            </Data>
          </xsl:if>
          <Data Column="package_id">
            <xsl:value-of select="$package_guid"/>
          </Data>
        </Row>
        <!--

        <xsl:call-template name="installprop_table_gen">
          <xsl:with-param name="props" select="installprop"/>
          <xsl:with-param name="role_id" select="@role_id"/>
        </xsl:call-template>
-->
      </xsl:for-each>
    </CustomTable>
  </xsl:template>

  <!-- must be called from the main template-->
  <xsl:template name="license_table_gen">
    <CustomTable Id="CO_LICENSE">
      <Column Id="id" Type="string" Category="Guid" Nullable="no" PrimaryKey="yes"/>
      <Column Id="license_text" Category="Text" Nullable="no" Type="string"/>
      <!-- should license_url be nullable? -->
      <Column Id="license_url" Category="Text" Nullable="yes" Type="string"/>
      <Column Id="role_id" Category="Guid" Nullable="no" Type="string" KeyTable="CO_ROLE" KeyColumn="1"/>
      <!--<xsl:apply-templates select="ms:node-set($role-union)" mode="licenses"/>-->
    </CustomTable>
  </xsl:template>

  <xsl:template match="in:SharedLib" mode="licenses">
    <Row>
      <Data Column="id">
        <xsl:value-of select="./in:License/@license_guid"/>
      </Data>
      <Data Column="license_text">
        <xsl:value-of select="./in:License"/>
      </Data>
      <Data Column="license_url">
        <xsl:value-of select="./in:License/@Url"/>
      </Data>
      <Data Column="role_id">
        <xsl:value-of select="@role_guid"/>
      </Data>
    </Row>
  </xsl:template>



  <xsl:template name="tag_table_gen">

    <xsl:param name="package_guid"/>
    <CustomTable Id="CO_TAGS">
      <Column Id="id" Category="Guid" Nullable="no" Type="string" PrimaryKey="yes"/>
      <Column Id="type" Category="Text" Nullable="no" Type="string"/>
      <Column Id="value" Category="Text" Nullable="no" Type="string"/>
      <Column Id="package_id" Category="Guid" Nullable="no" KeyTable="CO_PACKAGE" KeyColumn="1" Type="string"/>
      <xsl:for-each select="in:Tags/in:Tag">
        <Row>
          <Data Column="id">
            <xsl:value-of select="@tag_guid"/>
          </Data>
          <Data Column="type">
            <xsl:value-of select="@Type"/>
          </Data>
          <Data Column="value">
            <xsl:value-of select="@Value"/>
          </Data>
          <Data Column="package_id">
            <xsl:value-of select="$package_guid"/>
          </Data>
        </Row>
      </xsl:for-each>
    </CustomTable>
  </xsl:template>

  <xsl:template name="dependency_table_gen">
    <xsl:param name="package_guid"/>
    <CustomTable Id="CO_DEPENDENCY">
      <!--<Column Id="package_id" Category="Guid" Nullable="no" Type="string" />-->
      <!-- these can't be a foreign key for CO_PACKAGE because obviously that package isn't available-->
      <Column Id="dependency_id"  Category="Guid" Nullable="no" Type="string" PrimaryKey="yes"/>
      <xsl:for-each select="in:Dependencies/in:Dependency">
        <Row>
          <!--  <Data Column="package_id"><xsl:value-of select="$package_guid"/></Data>-->
          <Data Column="dependency_id">
            <xsl:value-of select="@package_id"/>
          </Data>
        </Row>
      </xsl:for-each>
    </CustomTable>
  </xsl:template>

  <xsl:template name="installprop_table_gen">
    <xsl:param name="roles"/>
    <CustomTable Id="CO_INSTALL_PROPERTIES">
      <Column Id="prop_id" Category="Guid" Type="string" Nullable="no" PrimaryKey="yes"/>
      <Column Id="type" Category="Text" Type="string" Nullable="no"/>
      <Column Id="link" Category="Text" Type="string" Nullable="no"/>
      <Column Id="target" Category="Text" Type="string" Nullable="no"/>
      <Column Id="role_id" Category="Guid" Nullable="no" Type="string" KeyTable="CO_ROLES" KeyColumn="1"/>

      <xsl:for-each select="$roles">
        <xsl:if test="in:InstallProperties">
          <xsl:for-each select="in:InstallProperties/in:Property">
            <Row>
              <Data Column="prop_id"><xsl:value-of select="@prop_guid"/></Data>
              <Data Column="type"><xsl:value-of select="@Type"/></Data>
              <Data Column="link"><xsl:value-of select="@Link"/></Data>
              <Data Column="target"><xsl:value-of select="@Target"/></Data>
              <Data Column="role_id"><xsl:value-of select="../../@role_guid"/></Data>
            </Row>
          </xsl:for-each>
        </xsl:if>
      </xsl:for-each>
    </CustomTable>
  </xsl:template>

  <xsl:template match="in:Publisher">
    <CustomTable Id="CO_PUBLISHER" >
      <Column Id="public_key_token" Nullable="no" PrimaryKey="yes" Category="Text" Type="string"/>
      <Column Id="name" Nullable="no" Category="Text" Type="string"/>
      <Column Id="location" Nullable="yes" Category="Guid" Type="string" KeyTable="CO_URLS" KeyColumn="1"/>
      <Column Id="email" Nullable="yes" Category="Text" Type="string"/>
      <Row>
        <Data Column="public_key_token">
          <xsl:value-of select="@PublicKeyToken"/>
        </Data>
        <Data Column="name">
          <xsl:value-of select="@Name"/>
        </Data>
        <xsl:if test="@location_id">
          <Data Column="location">
            <xsl:value-of select="@location_id"/>
          </Data>
        </xsl:if>
        <xsl:if test="@email">
          <Data Column="email">
            <xsl:value-of select="@email"/>
          </Data>
        </xsl:if>
      </Row>
    </CustomTable>
  </xsl:template>
 
  <!-- COAPPTABLES End-->
  
  <!-- Application Role Start -->

  <xsl:template match="in:AppRole">
    <xsl:param name="pubname"/>


    <xsl:param name="proddir_id" />
    <xsl:param name="name"/>
    <xsl:param name="version"/>
    <xsl:param name="arch"/>


    
      <xsl:apply-templates select="in:Files/*">
        <xsl:with-param name="version" select="$version" />
        <xsl:with-param name="role_arch" select="$arch"/>
        <xsl:with-param name="flavor" select="@Flavor"/>
      </xsl:apply-templates>
    

  </xsl:template>

  <xsl:template match="in:ExeFile">
    <xsl:param name="version"/>
    <xsl:param name="role_arch"/>
    <xsl:param name="flavor"/>

    <!-- if the assembly is 64-bit set win64 to yes!!-->
    <Component Guid="{@component_guid}" Id="{@component_id}">
      <xsl:if test="@Arch = 'x64'">
        <xsl:attribute name="Win64">yes</xsl:attribute>
      </xsl:if>

      <File Source="{@Src}" Vital="yes" KeyPath="yes">
        <xsl:attribute name="Name">
          <xsl:call-template name="ComplexFileName">
            <xsl:with-param name="name" select="@name"/>
            <xsl:with-param name="ext" select="@ext"/>
            <xsl:with-param name="version" select="$version"/>
            <xsl:with-param name="arch" select="@Arch"/>
          </xsl:call-template>
        </xsl:attribute>
      </File>
    </Component>

  </xsl:template>
  <!-- Application Role End -->
  
  <!-- DeveloperLib Role Start -->
  <xsl:template match="in:DeveloperLib">
    <xsl:param name="pubname"/>


    <xsl:param name="proddir_id" />
    <xsl:param name="name"/>
    <xsl:param name="version"/>
    <xsl:param name="arch"/>



    
      <xsl:apply-templates select="in:Includes/*">
        <xsl:with-param name="version" select="$version" />
        <xsl:with-param name="role_arch" select="$arch"/>
        <xsl:with-param name="flavor" select="@Flavor"/>
      </xsl:apply-templates>
    

  </xsl:template>



  <xsl:template match="in:Headers">
    <xsl:param name="version"/>
    <xsl:param name="role_arch"/>
    <xsl:param name="flavor"/>
    <Directory Name="include" Id="{@folder_id}">
      <xsl:apply-templates>
        <xsl:with-param name="version" select="$version" />
        <xsl:with-param name="role_arch" select="$role_arch"/>
      </xsl:apply-templates>
    </Directory>
  </xsl:template>

  <xsl:template match="in:ImportLibs">
    <xsl:param name="version"/>
    <xsl:param name="role_arch"/>
    <xsl:param name="flavor"/>
    <xsl:apply-templates mode="import">
      <xsl:with-param name="version" select="$version" />
      <xsl:with-param name="role_arch" select="$role_arch"/>
      <xsl:with-param name="flavor" select="$flavor"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="in:StaticLibs">
    <xsl:param name="version"/>
    <xsl:param name="role_arch"/>
    <xsl:param name="flavor"/>
    <xsl:apply-templates mode="static">
      <xsl:with-param name="version" select="$version" />
      <xsl:with-param name="role_arch" select="$role_arch"/>
      <xsl:with-param name="flavor" select="$flavor"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="in:LibFile" mode="import">
    <xsl:param name="version"/>
    <xsl:param name="role_arch"/>
    <xsl:param name="flavor"/>
    <Component Guid="{@component_guid}" Id="{@component_id}">
      <xsl:if test="@Arch = 'x64'">
        <xsl:attribute name="Win64">yes</xsl:attribute>
      </xsl:if>
      <File Source="{@Src}" Vital="yes">
        <xsl:attribute name="Name">
          <xsl:call-template name="ComplexFileName">
            <xsl:with-param name="name" select="@name"/>
            <xsl:with-param name="arch" select="@Arch"/>
            <xsl:with-param name="ext" select="@ext"/>
            <xsl:with-param name="compiler" select ="@Compiler"/>
            <xsl:with-param name="version" select="$version"/>
            <xsl:with-param name="flavor" select="$flavor"/>
          </xsl:call-template>
        </xsl:attribute>

      </File>

    </Component>

  </xsl:template>

  <xsl:template match="in:LibFile" mode="static">
    <xsl:param name="version"/>
    <xsl:param name="role_arch"/>
    <xsl:param name="flavor"/>
    <Component Guid="{@component_guid}" Id="{@component_id}">
      <xsl:if test="@Arch = 'x64'">
        <xsl:attribute name="Win64">yes</xsl:attribute>
      </xsl:if>
      <File Source="{@Src}" Vital="yes">
        <xsl:attribute name="Name">
          <xsl:call-template name="ComplexFileName">
            <xsl:with-param name="name" select="@name"/>
            <xsl:with-param name="arch" select="@Arch"/>
            <xsl:with-param name="ext" select="@ext"/>
            <xsl:with-param name="compiler" select ="@Compiler"/>
            <xsl:with-param name="version" select="$version"/>
            <xsl:with-param name="flavor" select="$flavor"/>
            <xsl:with-param name="static" select="'just something'"/>
          </xsl:call-template>
        </xsl:attribute>

      </File>

    </Component>

  </xsl:template>
  
  <!-- DeveloperLib Role End-->
  
  <!--SharedLib Role Start -->
  <xsl:template match="in:SharedLib">
    <xsl:param name="pubname"/>


    <xsl:param name="proddir_id" />
    <xsl:param name="name"/>
    <xsl:param name="version"/>
    <xsl:param name="arch"/>


    
      <xsl:apply-templates select="in:Files/*">
        <xsl:with-param name="version" select="$version" />
        <xsl:with-param name="role_arch" select="$arch"/>
        <xsl:with-param name="flavor" select="@Flavor"/>
      </xsl:apply-templates>
    

  </xsl:template>


  <xsl:template match="in:Assembly">
    <xsl:param name="version"/>
    <xsl:param name="role_arch" />
    <xsl:param name="flavor"/>
    <!-- if the assembly is 64-bit set win64 to yes!!-->
    <Component Guid="{@component_guid}" Id="{@component_id}">
      <xsl:if test="@Arch = 'x64'">
        <xsl:attribute name="Win64">yes</xsl:attribute>
      </xsl:if>

      <!-- setup assembly -->

      <!-- need to add compiler and flavor! -->
      <xsl:choose>
        <xsl:when test="@Arch = 'any'">
          <File Assembly="{@type}" Name="{concat(@name,@ext)}" AssemblyManifest="{@component_id}_assm" Id="{@component_id}_assm" Source="{@Src}" Vital="yes" KeyPath="yes">

          </File>
        </xsl:when>
        <xsl:otherwise>
          <File Assembly="{@type}" Name="{concat(@name,@ext)}" AssemblyManifest="{@component_id}_manifest" Source="{@Src}" Vital="yes" KeyPath="yes">

          </File>



          <!-- setup manifest -->

          <File Id="{concat(@component_id,'_manifest')}"  Vital="yes" Name="{concat(@name,@ext, '.manifest')}">

            <xsl:attribute name="Source">
              <xsl:value-of select="in:Manifest/@Src"/>
            </xsl:attribute>
          </File>

          <File Id="{concat(@component_id,'_cat')}"  Name="{concat(@name,@ext, '.cat')}" Vital="yes">

            <xsl:attribute name="Source">
              <xsl:value-of select="in:CatFile/@Src"/>
            </xsl:attribute>
          </File>
        </xsl:otherwise>
      </xsl:choose>
      

      <!-- set up folder and file-->
      <xsl:apply-templates select="in:File" mode="nocomponent"/>
    </Component>
  </xsl:template>

  <xsl:template match="in:PolicyAssembly">
    <xsl:param name="version"/>
    <xsl:param name="role_arch" />
    <xsl:param name="flavor"/>
    <!-- if the assembly is 64-bit set win64 to yes!!-->
    <Component Guid="{@component_guid}" Id="{@component_id}">
      <xsl:choose>
        <xsl:when test="@type = '.net'">
          <File Id="{concat(@component_id, '_assm')}" Assembly="{@type}" Vital="yes" Name="{concat(@name, @ext)}" AssemblyManifest="{concat(@component_id, '_assm')}" KeyPath="yes"
                Source="{@Src}"/>
          <File Vital="yes" Source="{in:Manifest/@Src}"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:if test="@Arch = 'x64'">
            <xsl:attribute name="Win64">yes</xsl:attribute>
          </xsl:if>
          <File Id="{concat(@component_id,'_manifest')}" Assembly="{@type}"  Vital="yes" Name="{concat(in:Manifest/@name,in:Manifest/@ext)}"  AssemblyManifest="{@component_id}_manifest" KeyPath="yes">
            <xsl:attribute name="Source">
              <xsl:value-of select="in:Manifest/@Src"/>
            </xsl:attribute>
          </File>

          <File Id="{concat(@component_id,'_cat')}"  Name="{concat(in:CatFile/@name,in:CatFile/@ext)}" Vital="yes">

            <xsl:attribute name="Source">
              <xsl:value-of select="in:CatFile/@Src"/>
            </xsl:attribute>
          </File>
        </xsl:otherwise>
      </xsl:choose>
      
    </Component>
  </xsl:template>

 
  <!-- SharedLib Role End-->
</xsl:stylesheet>
